;============================================================
;
; DIS386 macros and equates
; Copyright (c) 1992-2003,2004 Alessandro Scotti
; http://www.ascotti.org/
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;============================================================

; Prefix opcodes
PFX_RECORD      RECORD  PFX_UNUSED:06,          \
                        PFX_ADDRESS32:1,        \       ; Address size
                        PFX_OPERAND32:1,        \       ; Operand size
                        PFX_LOCK:1,             \       ; Lock
                        PFX_REPE:1, PFX_REPNE:1,\       ; Rep/repe/repne
                        PFX_OVRD:1,             \       ; Segment override
                        PFX_OVRDSEG:4                   ; Segment code

; Configuration flags
CNF_RECORD      RECORD  CNF_UNUSED:08,          \
                        CNF_INDENTJMP:1,        \       ; Indent jumps on/off
                        CNF_UPCASE:1,           \       ; Uppercase on/off
                        CNF_SOURCE:1,           \       ; Source mode on/off
                        CNF_USE32:1,            \       ; 16/32 bit offset
                        CNF_CPU:4                       ; CPU type

; CPU types
CNF_CPUALL      EQU     0               ; Enables all opcodes
CNF_CPU8086     EQU     1               ;   "    only 8086 opcodes
CNF_CPU80186    EQU     2               ;   "    up to 80186 opcodes
CNF_CPU80286    EQU     3               ;   "    up to 80286 opcodes
CNF_CPU80386    EQU     4               ;   "    up to 80386 opcodes
CNF_CPU80486    EQU     5               ;   "    up to 80486 opcodes

; SUB_Decode return codes                        
RET_PREFIX      EQU     1
RET_ERROR       EQU     2

BUFSIZE         EQU     80              ; Size of output buffer
TABSIZE         EQU     8               ; Size of TAB (must be power of two)
TAB             EQU     09h             ; ASCII code for TAB

; Control structure
DIS_CONTROL     STRUC
; Fields filled by user
        C_Flags         DW      ?       ; Control flags
        C_VirtualSeg    DW      ?       ; Virtual instruction segment
        C_VirtualOfs    DD      ?       ; Virtual instruction pointer
        C_CodePtr       DD      ?       ; Pointer to instruction to decode
        C_BufferPtr     DD      ?       ; Pointer to output buffer
        C_ListSize      DW      ?       ; Number of chars reserved for listing
; Fields filled by disassembler
        C_Name          DB      ?       ; Opcode name index
        C_Arg1          DB      ?       ; First argument
        C_Arg2          DB      ?       ; Second argument
        C_Arg3          DB      ?       ; Third argument
        C_Prefix        PFX_RECORD ?    ; Prefixes of opcode
        C_Reserved      DB      10 DUP(?)
DIS_CONTROL     ENDS

;------------------------------------------------------------
; Macro to push registers, variables or flags onto the stack
; Usage: .push "loc16"[,"loc16"...]
; where "loc16" is a 16-bit register, a word-sized variable or the
; keyword "FLAGS".
; Exmpl: .push ax, flags, var1
;        .pop  ax, flags, var1
;
.push   MACRO   r0, r1, r2, r3, r4, r5, r6, r7, r8, r9, r10
    IFNB <r10>
        .ERROR <.PUSH has more than 10 arguments>
    ENDIF
    IRP $reg, <r0, r1, r2, r3, r4, r5, r6, r7, r8, r9>
        IFB <$reg>                      ;; Is argument blank?
            EXITM                       ;; Yes, exit
        ELSEIFIDNI <$reg>, <FLAGS>      ;; Is argument the keyword "FLAGS"?
            pushf                       ;; Yes, push flags
        ELSE
            push    $reg                ;; Push argument
        ENDIF
    ENDM
ENDM

;------------------------------------------------------------
; Macro to pop registers, variables or flags from the stack
; Usage: .pop "loc16"[,"loc16"...]
; where "loc16" is a 16-bit register, a word-sized variable or the
; keyword "FLAGS".
; Exmpl: .push ax, flags, var1
;        .pop  ax, flags, var1
;
.pop    MACRO   r0, r1, r2, r3, r4, r5, r6, r7, r8, r9, r10
    IFNB <r10>
        .ERROR <.POP has more than 10 arguments>
    ENDIF
    IRP $reg, <r9, r8, r7, r6, r5, r4, r3, r2, r1, r0>
        IFNB <$reg>                     ;; Is argument non-blank?
            IFIDNI <$reg>, <FLAGS>      ;; Yes, is it the keyword "FLAGS"?
                popf                    ;; Yes, pop flags
            ELSE
                pop     $reg            ;; Pop argument
            ENDIF
        ENDIF
    ENDM
ENDM

;------------------------------------------------------------
; Macro to write a character into the output buffer
.putc   MACRO   char
        IFNB <char>
            mov     al, char
        ENDIF
        call    SUB_PutC
ENDM
